@@ -1,7 +1,13 @@
 #!/usr/bin/env python3
 
-import sys, os, subprocess, hashlib, tempfile, shutil
+import sys, os, subprocess, hashlib
 
+def cmd(command):
+	print (command)
+	global process
+	process.stdin.write ((':'.join (command) + '\n').encode ("utf-8"))
+	process.stdin.flush ()
+	return process.stdout.readline().decode ("utf-8").strip ()
 
 args = sys.argv[1:]
 
@@ -10,25 +16,16 @@ if len (args) and args[0] == "--reference":
 	reference = True
 	args = args[1:]
 
+have_freetype = bool(int(os.getenv ('HAVE_FREETYPE', '1')))
+
 if not args or args[0].find('hb-shape') == -1 or not os.path.exists (args[0]):
 	sys.exit ("""First argument does not seem to point to usable hb-shape.""")
 hb_shape, args = args[0], args[1:]
 
-def cmd(command):
-	process = subprocess.Popen ([hb_shape, '--batch'],
+process = subprocess.Popen ([hb_shape, '--batch'],
 			    stdin=subprocess.PIPE,
 			    stdout=subprocess.PIPE,
-			    stderr=subprocess.PIPE)
-	process.stdin.write ((' '.join (command) + '\n').encode ("utf-8"))
-	process.stdin.flush ()
-	ret_stdout = None
-	ret_stderr = None
-	ret_stdout, ret_stderr = process.communicate ()
-	if ret_stdout is not None:
-		ret_stdout = ret_stdout.decode ("utf-8").strip ()
-	if ret_stderr is not None:
-		ret_stderr = ret_stderr.decode ("utf-8").strip ()
-	return (ret_stdout, ret_stderr)
+			    stderr=sys.stdout)
 
 passes = 0
 fails = 0
@@ -65,6 +62,7 @@ for filename in args:
 			continue
 
 		fontfile, options, unicodes, glyphs_expected = line.split (":")
+		options = options.split ()
 		if fontfile.startswith ('/') or fontfile.startswith ('"/'):
 			if os.name == 'nt': # Skip on Windows
 				continue
@@ -100,41 +98,29 @@ for filename in args:
 			print ('%s "%s" %s %s --unicodes %s' %
 					 (hb_shape, fontfile, ' '.join(extra_options), options, unicodes))
 
-		# hack to support fonts with space on run-tests.py, after several other tries...
-		if ' ' in fontfile:
-			new_fontfile = os.path.join (tempfile.gettempdir (), 'tmpfile')
-			shutil.copyfile(fontfile, new_fontfile)
-			fontfile = new_fontfile
-
-		glyphs1 = cmd ([hb_shape, "--font-funcs=ft",
-			fontfile] + extra_options + ["--unicodes",
-			unicodes] + (options.split (' ') if options else []))
-
-		if glyphs1[1] is not None or glyphs1[1] != '':
-			check_string = hb_shape[hb_shape.find(os.path.sep) + 1:] + \
-			               ': Unknown font function implementation `ft\''
-			if glyphs1[1].startswith(check_string):
-				skips += 1
-				print ("Skipping test due to lack of FreeType support")
-				continue
-
-		glyphs2 = cmd ([hb_shape, "--font-funcs=ot",
-			fontfile] + extra_options + ["--unicodes",
-			unicodes] + (options.split (' ') if options else []))
-
-		if glyphs1[0] != glyphs2[0] and glyphs_expected != '*':
-			print ("FT funcs: " + glyphs1[0], file=sys.stderr)
-			print ("OT funcs: " + glyphs2[0], file=sys.stderr)
-			fails += 1
-		else:
-			passes += 1
-
-		if reference:
-			print (":".join ([fontfile, options, unicodes, glyphs1[0]]))
+		if "--font-funcs=ft" in options and not have_freetype:
+			skips += 1
 			continue
 
-		if glyphs1[0].strip() != glyphs_expected and glyphs_expected != '*':
-			print ("Actual:   " + glyphs1[0], file=sys.stderr)
+		if "--font-funcs=ot" in options or not have_freetype:
+			glyphs1 = cmd ([hb_shape, fontfile, "--font-funcs=ot"] + extra_options + ["--unicodes", unicodes] + options)
+		else:
+			glyphs1 = cmd ([hb_shape, fontfile, "--font-funcs=ft"] + extra_options + ["--unicodes", unicodes] + options)
+			glyphs2 = cmd ([hb_shape, fontfile, "--font-funcs=ot"] + extra_options + ["--unicodes", unicodes] + options)
+
+			if glyphs1 != glyphs2 and glyphs_expected != '*':
+				print ("FT funcs: " + glyphs1, file=sys.stderr)
+				print ("OT funcs: " + glyphs2, file=sys.stderr)
+				fails += 1
+			else:
+				passes += 1
+
+		if reference:
+			print (":".join ([fontfile, " ".join(options), unicodes, glyphs1]))
+			continue
+
+		if glyphs1.strip() != glyphs_expected and glyphs_expected != '*':
+			print ("Actual:   " + glyphs1, file=sys.stderr)
 			print ("Expected: " + glyphs_expected, file=sys.stderr)
 			fails += 1
 		else:
